#!bioTk -f

# $Id: tkmap.tcl,v 1.9 1996/05/20 16:10:48 humberto Exp $
# bioTk applet to draw genetic maps.
# Copyright (C) 1996 Humberto Ortiz Zuazaga and Rosemarie Plaetke

## Set up the global variables.
global Width Height Maxrange Intervals Map Locus Font BoldFont Fontsize Break1 Frame1 Frame2 Conf Framework Margin
set Width 600
set Height 600
set Fontsize 12
set Font -*-helvetica-medium-r-normal-*-*-[expr int([expr $Fontsize * 10])]-*-*-*-*-*-*
set BoldFont -*-helvetica-bold-r-normal-*-*-[expr int([expr $Fontsize * 10])]-*-*-*-*-*-*
set Maxrange 0
set Margin [expr $Width / 32]

set Frame1 0
set Frame2 0
set Break1 0
set Conf 0

# Parse in the labels and distances. Return the maximum range of the map.
proc makeframe {file} {
    global Map Intervals Maxrange Frame1 Frame2 Conf Framework

    bioTk_LogoMotion . 1
    .logo configure -fg red

    catch {unset Map Intervals Maxrange Locus}
    
    set Frame1 1

    set file [open $file r]
    set place 0.0
    set Intervals {}
    while {[gets $file line] >= 0} {
	if {[regexp {^[ \t]*($|\#)} $line]} {
	    continue
	}
	update
	set marker [lindex $line 0]
	set increment [lindex $line 1]
	if {[regexp {\*$} $marker]} {
	    set marker [string trimright $marker *]
	    set Framework($marker) 1
	} else {
	    set Framework($marker) 0
	}	
	set Map($marker) $place
	if {2 == [llength $line]} {
	    lappend Intervals [list $increment \
		    [expr $place + [expr $increment / 2.0]]]
	    set place [expr $place + $increment]
	}
    }
    close $file

    set Maxrange $place
    drawmap

    bioTk_LogoMotion . 0
    .logo configure -fg black

}

# Parse in the labels, construct a breakpoint map evenly spaced.
proc makebreak {file} {
    global Map Maxrange Break1 Conf Framework

    bioTk_LogoMotion . 1
    .logo configure -fg red

    catch {unset Map Maxrange Locus}
    
    set Break1 1

    set file [open $file r]
    set place 0.0
    while {[gets $file line] >= 0} {
	if {[regexp {^[ \t]*($|\#)} $line]} {
	    continue
	}
	update
	set marker [lindex $line 0]
	if {[regexp {\*$} $marker]} {
	    set marker [string trimright $marker *]
	    set Framework($marker) 1
	} else {
	    set Framework($marker) 0
	}	
	set Map($marker) $place
	set place [expr $place + 1]
    }
    close $file

    set Maxrange [expr $place - 1]
    drawbreak

    bioTk_LogoMotion . 0
    .logo configure -fg black

}

# Read in the confidence intervals.
proc makelocus {file} {
    global Locus Frame1 Frame2 Conf

    bioTk_LogoMotion . 1
    .logo configure -fg red

    catch {unset Locus}

    set Conf 1

    set file [open $file r]
    while {[gets $file line] >= 0} {
	if {[regexp {^[ \t]*($|\#)} $line]} {
	    continue
	}
	update
	if {3 == [llength $line]} {
	    set marker [lindex $line 0]
	    set start [lindex $line 1]
	    set end [lindex $line 2]
	    set Locus($marker) [list [list $start $end]]
	}
    }
    close $file
    drawconf

    bioTk_LogoMotion . 0
    .logo configure -fg black

}

### Mapping function for confidence intervals
proc locus g {
    global Map Locus Frame1 Frame2 Conf

    set span {}
    foreach m $g {
	lappend span $Map($m)
    }
    return $span
}

# Dump out the map to an Encapsulated PostScript file.
proc printmap f {
    update
    .c postscript -file $f
    bell
}

## Draw the framework map on the canvas
proc drawmap {} {
    global Maxrange Map Intervals Width Height Font BoldFont Frame1 Frame2 Conf Framework

    bioTk_LogoMotion . 1
    .logo configure -fg red

    .c delete sizes
    .c delete genetic
# delete the axis too, but I don't know how.
    bioTk_MapReset sizes
    bioTk_MapReset genetic
    bioTk_MapUnset sizes
    bioTk_MapUnset genetic


    ### Look! the left and right coords are reversed, so labels print 
    ### to the LEFT of the axis.  Don't mess with this!

    set size(left) [expr $Width / 8]
    set size(right) [expr $Width / 16]
    set size(top) [expr $Height / 16]
    set size(bottom) [expr $Height - $size(top)]
    
    bioTk_Map .c sizes $size(left) $size(top) $size(right) $size(bottom) \
	    -orientation vertical \
	    -range [list 0 $Maxrange] -linewidth 1 \
	    -labelfont $Font
    
    set genetic(left) $size(left)
    set genetic(right) [expr $genetic(left) + [expr $Width / 8]]
    set genetic(top) $size(top)
    set genetic(bottom) $size(bottom)
    
    bioTk_Map .c genetic $genetic(left) $genetic(top) \
	    $genetic(right) $genetic(bottom) \
	    -orientation vertical \
	    -range [list 0 $Maxrange] -linewidth 1 \
	    -labelfont $Font
    
    bioTk_MapAxis genetic
    
    # Build the framework
    foreach marker [array names Map] {
	update
	if {$Framework($marker)} {
	    bioTk_MapObject genetic $Map($marker) \
		-at [list $genetic(left) [expr $genetic(left) + [expr $Width / 32]]] \
		-label $marker -anchor apart -labelcolor black -labelfont $BoldFont
	} else {
	    bioTk_MapObject genetic $Map($marker) \
		-at [list $genetic(left) [expr $genetic(left) + [expr $Width / 32]]] \
		-label $marker -anchor apart -labelcolor black
	}
    }
    
    # And the "rung" sizes
    foreach int $Intervals {
	update
	bioTk_MapObject sizes [lindex $int 1] -label [lindex $int 0] \
		-anchor apart \
		-width 0 \
		-at [list [expr $size(left) - 5] [expr $size(right) + [expr $Width / 32]]]
    }
    
    bioTk_LogoMotion . 0
    .logo configure -fg black

}

## Draw the breakpoint map on the canvas
proc drawbreak {} {
    global Maxrange Map Width Height Font BoldFont Frame1 Frame2 Conf Framework

    bioTk_LogoMotion . 1
    .logo configure -fg red

    .c delete all
    bioTk_MapReset genetic
    bioTk_MapReset sizes

    bioTk_MapUnset genetic
    bioTk_MapUnset sizes


    ### Look! the left and right coords are reversed, so labels print 
    ### to the LEFT of the axis.  Don't mess with this!

    set size(left) [expr $Width / 8]
    set size(right) [expr $Width / 16]
    set size(top) [expr $Height / 16]
    set size(bottom) [expr $Height - $size(top)]
    
    set genetic(left) $size(left)
    set genetic(right) [expr $genetic(left) + [expr $Width / 8]]
    set genetic(top) $size(top)
    set genetic(bottom) $size(bottom)
    
    bioTk_Map .c genetic $genetic(left) $genetic(top) \
	    $genetic(right) $genetic(bottom) \
	    -orientation vertical \
	    -range [list 0 $Maxrange] -linewidth 1 \
	    -labelfont $Font
    
    bioTk_MapAxis genetic
    
    # Build the framework
    foreach marker [array names Map] {
	update
	if {$Framework($marker)} {
	    bioTk_MapObject genetic $Map($marker) \
		-at [list $genetic(left) [expr $genetic(left) + [expr $Width / 32]]] \
		-label $marker -anchor apart -labelcolor black -labelfont $BoldFont
	} else {
	    bioTk_MapObject genetic $Map($marker) \
		-at [list $genetic(left) [expr $genetic(left) + [expr $Width / 32]]] \
		-label $marker -anchor apart -labelcolor black
	}
    }
    
    bioTk_LogoMotion . 0
    .logo configure -fg black

}

## Draw the confidence intervals on the canvas
proc drawconf {} {
    global Maxrange Locus Width Height Font Frame1 Frame2 Conf Break1 Margin

    bioTk_LogoMotion . 1
    .logo configure -fg red

    .c delete confidence
    bioTk_MapReset confidence
    bioTk_MapUnset confidence


    if {!$Frame1 && !$Break1} {
	bioTk_Dialog "Please Load a Framework map first."
	return
    }

    set confidence(left) [expr [expr $Width / 4] + $Margin]
    set confidence(right) [expr $Width - [expr $Width / 16]]
    set confidence(top) [expr $Height / 16]
    set confidence(bottom) [expr $Height - [expr $Height / 16]]
    
    bioTk_Map .c confidence $confidence(left) $confidence(top) \
	    $confidence(right) $confidence(bottom) \
	    -orientation vertical \
	    -range [list 0 $Maxrange] -linewidth 1 \
	    -labelfont $Font
    
    # Add the confidence intervals
    foreach gene [array names Locus] {
	update
	bioTk_MapObject confidence $Locus($gene) -mapping locus \
		-label $gene -anchor e -width 2
    }
    
    bioTk_LogoMotion . 0
    .logo configure -fg black

}

## Edit the default settings
proc edit {} {
    global Width Height Font Fontsize Frame1 Frame2 Conf Margin

    toplevel .edit
    wm title .edit "Preferences"

    pack [frame .edit.width]
    pack [label .edit.width.label -text "Width"] -side left
    pack [entry .edit.width.entry -textvariable Width] -side left

    pack [frame .edit.height]
    pack [label .edit.height.label -text "Height"] -side left
    pack [entry .edit.height.entry -textvariable Height] -side left

    pack [frame .edit.margin]
    pack [label .edit.margin.label -text "Margin"] -side left
    pack [entry .edit.margin.entry -textvariable Margin] -side left

    pack [frame .edit.font]
    pack [label .edit.font.label -text "Font size"] -side left
    pack [entry .edit.font.entry -textvariable Fontsize] -side left

    pack [frame .edit.buttons]
    pack [button .edit.buttons.apply -text "Apply" -command {destroy .edit; apply_prefs}] -side left
    pack [button .edit.buttons.quit -text "Cancel" -command {destroy .edit}] -side left
}

proc apply_prefs {} {
    global Width Height Font Maxrange Fontsize Frame1 Conf Frame2 Break1 BoldFont Margin

    set Font -*-helvetica-medium-r-normal-*-*-[expr int([expr $Fontsize * 10])]-*-*-*-*-*-*
    set BoldFont -*-helvetica-bold-r-normal-*-*-[expr int([expr $Fontsize * 10])]-*-*-*-*-*-*

    if {"default" == $Margin} {
	set Margin [expr $Width / 32]
    }

    .c delete all
    .c configure -width $Width -height $Height
    if {$Frame1} {
	drawmap
    }
    if {$Break1} {
	drawbreak
    }
    if {$Conf} {
	drawconf
    }
}

###
# Begin main program
###
wm  title . "TkMap"
wm iconname . "tkmap"
wm iconbitmap . @$env(BIOTK_LIBRARY)/bitmaps/chromosome.bit
wm iconmask . @$env(BIOTK_LIBRARY)/bitmaps/chromosome.mask

bioTk_MenuBar . {
{File "File operations menu:  Load datafiles, print the map or exit the program." cascade {
 {Load "Load menu: Load the framework map or the confidence intervals." cascade {
  {"Linear Map" "" command {bioTk_FileBox -command makeframe}}
  {"Breakpoint Map" "" command {bioTk_FileBox -command makebreak}}
  {"Confidence intervals" "" command {bioTk_FileBox -command makelocus}}}}
 {Print "Print dialog, can print to a file or PostScript printer in several modes." command {bioTk_PrintBox .c}}
 separator
 {Quit "Exit TkMap" command {exit}}}}   
{Preferences "Set up some user preferences" command edit}
{Help "Activate balloon help, or browse the man page." cascade {
 {"Balloon Help" "Toggle the ballon help system on" command bioTk_StartHelp}
 {Manual "Browse the tkmap(1) man page." command {bioTk_ShowManual tkmap}}}}}

### Set up the drawing area.
pack [canvas .c -background #eff -width $Width -height $Height] -fill both -expand t

#bind .c <Configure> {
#    set Width %w
#    set Height %h
#
#    .c delete all
#    bioTk_MapReset confidence
#    bioTk_MapUnset confidence
#    bioTk_MapReset sizes
#    bioTk_MapReset genetic
#    bioTk_MapUnset sizes
#    bioTk_MapUnset genetic
#    if {$Frame1} {
#	drawmap
#    }
#    if {$Conf} {
#	drawconf
#    }
#}

catch {source .tkmaprc}
#wm configure

### End main.